/**
 * The <code>Actor</code> class is used to get an abstraction of the
 * plain text commands so that the can be used via methods. The methods
 * compose the commands and send them via the communication link.
 *
 * @author <a href="mailto:kuehne@pi4.informatik.uni-mannheim.de">Gerald Kuehne</a>
 * @author <a href="mailto:beier@pi4.informatik.uni-mannheim.de">Markus Beier</a>
 * @version 1.0
 */
public class Actor
{
    // reference to the communication module
    private Communication communication;

    /**
     * Creates a new <code>Actor</code> instance.
     *
     * @param communication a <code>Communication</code> value defining the
     * communication link to the server via which the commands can be
     * sent
     * @see Communication
     */
    public Actor(Communication communication)
    {
        this.communication = communication;
    }

    /**
     * Registers itself to to the server. Protocol version 7.0 is used by default.
     *
     * @param teamName The name of the team
     * the player should join or create if it does not already exist and there
     * less than two teams already present on the field
     */
    public void init(String teamName)
    {
        communication.send(new String("(init " + teamName + "(version 7.0))"));
    }

    /**
     * Gives a running impulse
     *
     * @param power a <code>double</code> value containing the power of the
     * impulse
     */
    public void dash(double power)
    {
        communication.send(new String("(dash " + power + ")"));
    }

    /**
     * Kicks into a given direction with given power
     *
     * @param power a <code>double</code> value specifying the power of the kick
     * @param direction a <code>double</code> value giving the direction of the kick
     */
    public void kick(double power, double direction)
    {
        communication.send(new String("(kick " + power + " " + direction + ")"));
    }

    /**
     * Places the player to a given position on the field. Only possible:
     * <ul>
     * <li> before kick off
     * <li> after a goal
     * </ul>

     * @param x gives the position in x direction.
     * The position can vary from -52.5 to 52.5. The unit is meter.
     * Negative values stand for the own half of the field, positive values stand for
     * the opponent's half. The value itself gives the distance from the mid-line.
     * @param y gives the position in y direction.
     * The position can vary from -34.0 to 34.0. The unit is meter.
     * Negative values stand for the left side of the field when looked in direction of
     * the opponent's goal.Positive value stand for the right side.
     * The value itself gives the distance from the line going through the center of both
     * goal and the middle point of the field.
     */
    public void move(double x, double y)
    {
        communication.send(new String("(move " + x + " " + y + ")"));
    }

    /**
     * Turns the body of the client on the field.
     *
     * @param moment is the angle in degree you want to turn.
     * The orientation is clockwise so that positive angles mean right turns. The absolute
     * value of the angel must be smaller than 180 degrees. Otherwise the the server will
     * take the turn as a 180 degree turn. So it should be assured that the angle are
     * normalized before usage
     */
    public void turn(double moment)
    {
        communication.send(new String("(turn " + moment + ")"));
    }
}
