/**
 * This is the central class of the Simpleclient. Here, all visual information
 * is evaluated and converted into decisions resp. actions to:
 * <ul>
 * <li> find out the positions of the ball and the goal
 * <li> turn towards the ball
 * <li> run to the ball
 * <li> kick it into the direction of the right goal
 * </ul>
 *
 * So one limitation is, that the client only can play from left to right
 *
 * @author <a href="mailto:kuehne@pi4.informatik.uni-mannheim.de">Gerald Kuehne</a>
 * @author <a href="mailto:beier@pi4.informatik.uni-mannheim.de">Markus Beier</a>
 * @version 1.0
 */
public class Player extends Thread
{
    private Actor  actor;
    private Sensor sensor;
    private String teamName;

    final private int NOT_CONNECTED = 0;
    final private int CONNECTED     = 1;
    final private int PLAYING       = 2;

    private int status = NOT_CONNECTED;

    private ObjectInfo theBall = null;
    private ObjectInfo theGoal = null;

    private int time = -1;


    /**
     * Creates a new <code>Player</code> instance.
     *
     * @param servername a <code>String</code> value specifying the the name of the server
     * @param serverport an <code>int</code> value specifying the port on which the server
     *                   listens for incoming packets
     * @param teamName a <code>String</code> value is the name of the team you want to join/create
     */
    public Player(String servername, int serverport, String teamName)
    {
        this.teamName = teamName;

        Communication communication = new Communication(servername, serverport);
        actor   = new Actor(communication);
        sensor  = new Sensor(communication);
    }


    /**
     * This method had to overwritten from <code>Thread</code>. It is
     * executed when <code>start()</code> is called for the <code>Player</code>
     * Thread
     *
     */
    public void run()
    {
        Event event = null;
        // connect to server
        actor.init(teamName);
        // and play
        while (true) {
            // receive event - block until one arrives
            event = sensor.receive();
            for (int i = 0; i < event.objectInfoList.size(); i++) {
                // check for the ball and the oponents' goal
                ObjectInfo objectInfo = (ObjectInfo) event.objectInfoList.get(i);
                if (objectInfo.objectName.startsWith("b")) {
                    theBall = objectInfo;
                } else if (objectInfo.objectName.startsWith("g r")) {
                    theGoal = objectInfo;
                }
            }

            // action ...
            if (event instanceof BodyEvent) {
                play();
            }
        }
    }

    /**
     * Processes the information gained and transforms it into a decision/action.
     * It should only be called once per 100ms period, so its proper execution on
     * the server is assured
     *
     */
    private void play()
    {
        switch (status) {
        case NOT_CONNECTED:
            actor.init(teamName);
            status = CONNECTED;
            break;
        case CONNECTED:
            // set the player to a random position DEEP in the left field
            actor.move(-Math.random() * 22.5-30, Math.random() * 2 * 20.0 - 20.0);
            status = PLAYING;
            break;
        case PLAYING:

            // already found the ball?
            if (theBall == null) {
                // no, look around
                actor.turn(40);
            } else {
                // we found the ball
                // turn to it if necessary but only if new ball info is available
                if (Math.abs(theBall.direction) > 1.0 && theBall.time != time) {
                    actor.turn(theBall.direction);
                    // remember the time of the info the action was based on
                    time = theBall.time;
                    System.out.println("Found the ball and turned to it");
                // check the distance
                } else if (theBall.distance > 0.5) {
                    actor.dash(10 * theBall.distance);
                    System.out.println("Here we go");
                // look for the goal
                } else if (theGoal == null) {
                    actor.turn(40);
                // kick
                } else {
                    actor.kick(50, theGoal.direction);
                }
            }
            break;
        }
    }

    /**
     * Starts the client
     *
     * @param args The arguments passed must specify <code>SERVERNAME PORT TEAMNAME</code> in this order.
     */
    public static void main (String[] args) {
        if (args.length != 3) {
            System.out.println("Usage : java Player SERVERNAME PORT TEAMNAME");
            System.exit(1);
        }

        Player player = new Player(args[0], Integer.parseInt(args[1]), args[2]);
        player.start();
    }
}
