/**
 * The class <code>Sensor</code> is used to preprocess incoming messages.
 * <BR> Here they are received and classified as <code>see</code>,
 * <code>hear</code> or other messages and forwarded to the respective
 * event classes
 *
 * @author <a href="mailto:kuehne@pi4.informatik.uni-mannheim.de">Gerald Kuehne</a>
 * @author <a href="mailto:beier@pi4.informatik.uni-mannheim.de">Markus Beier</a>
 * @version 1.0
 */
public class Sensor
{
    private Communication communication;

    /**
     * Creates a new <code>Sensor</code> instance.
     *
     * @param communication a <code>Communication</code> value containing the
     * Communication class through wich the messages can be received from the server
     */
    public Sensor(Communication communication)
    {
        this.communication = communication;
    }

    /**
     * Receives incoming messages from the server. They
     * are automatically classified and passed on to the correct parsing/representation
     * classes. <BR>
     * NOTE: The <code>receive()</code> method is blocking because is uses the blocking
     * <code>receive()</code> method of the <code>Communication</code> class. Therefor
     * the thread will be interrupted until a new message arrives.
     * @see Communication
     *
     * @return an <code>Event</code> object containing the parsed information
     * of the read message
     */
    public Event receive()
    {
        String msg;
        Event event = null;

        while (event == null) {
            // receive server message - block until one arrives
            msg = communication.receive();

            if (msg.startsWith("(see")) {
                // it's a visual ("see") or a body ("sense body") event
                event = new VisualEvent(msg);
            } else if (msg.startsWith("(sense_body")) {
                // it's a body ("sense_body") event
                event = new BodyEvent();
            }
        }

        return event;
    }

}
