import java.util.StringTokenizer;

/**
 * This class parses all messages regarding visual information of the game,
 * i.e. the <code>see</code> messages, and represents the information in
 * a list of <code>ObjectInfo</code>
 *
 * @author <a href="mailto:kuehne@pi4.informatik.uni-mannheim.de">Gerald Kuehne</a>
 * @author <a href="mailto:beier@pi4.informatik.uni-mannheim.de">Markus Beier</a>
 * @version 1.0
 */
public class VisualEvent extends Event
{
    /**
     * Creates a new <code>VisualEvent</code> instance, extracts the information contained
     * and stores it in a list of <code>ObjectInfo</code>
     * @see ObjectInfo
     *
     * @param message a <code>String</code> value which contains the
     * <code>see</code> message of the <code>rcssserver</code> to be
     * parsed and stored in the instance
     */
    public VisualEvent(String message)
    {
        // use parantheses as string delimiters
        StringTokenizer	tokenizer = new StringTokenizer(message, "()");

        // extract the time stamp
        // - nextToken returns something like "see 15 "
        // - use substring and trim whitespaces
        // Example:
        // The received String is:
        // (see 0 ((g r) 66.7 34) ((b) 20.3 5))

        // The first Token extracted would be:
        // "see 0 " (the blank at the end is included)
        // substring(4) returns "0 "; trim() removes leading and trailing blanks
        // so it returns "0"
        int time = Integer.parseInt(tokenizer.nextToken().substring(4).trim());

        while (tokenizer.hasMoreTokens()) {
            // create a new object info
            ObjectInfo objectInfo = new ObjectInfo();
            // set the time of the current event
            objectInfo.time = time;
            // get the object name
            // in the example the next Token would be:
            // "g r" for the right goal
            objectInfo.objectName = tokenizer.nextToken();

            switch (objectInfo.objectName.charAt(0)) {
                // we only handle ball and goal objects ...
            case 'b':
            case 'g':
                // the next token of the example in case of "g r" it would be:
                // " 66.7 34"
                // to parse this, we need a new Tokenizer taking space as a delimiter
                StringTokenizer st   = new StringTokenizer(tokenizer.nextToken(), " ");
                objectInfo.distance  = Double.parseDouble(st.nextToken());
                objectInfo.direction = Double.parseDouble(st.nextToken());
                break;
                // and ignore the rest
            case 'p':
            case 'f':
            case 'l':
                // in case of a not handled object, "eat" the following Tokens
                tokenizer.nextToken();
                tokenizer.nextToken();
                break;
            default:
                break;
            }

            objectInfoList.add(objectInfo);
        }
    }
}
